@@ -1,10 +1,7 @@
 """bareclient"""
 
 from .client import HttpClient
-from .session import HttpSession
-from .unbound_session import HttpUnboundSession
 from .helpers import (
-    request,
     get,
     get_text,
     get_json,
@@ -12,19 +9,19 @@ from .helpers import (
     post_text,
     post_json
 )
-from .constants import DEFAULT_PROTOCOLS, DEFAULT_DECOMPRESSORS
+from .constants import DEFAULT_PROTOCOLS
+from .middleware import HttpClientMiddlewareCallback, HttpClientCallback
 from .ssl_contexts import (
     create_ssl_context,
     create_ssl_context_with_cert_chain,
     DEFAULT_CIPHERS,
     DEFAULT_OPTIONS
 )
+from .request import Request
+from .response import Response
 
 __all__ = [
     'HttpClient',
-    'HttpSession',
-    'HttpUnboundSession',
-    'request',
     'get',
     'get_text',
     'get_json',
@@ -36,5 +33,8 @@ __all__ = [
     'DEFAULT_PROTOCOLS',
     'DEFAULT_CIPHERS',
     'DEFAULT_OPTIONS',
-    'DEFAULT_DECOMPRESSORS'
+    'Request',
+    'Response',
+    'HttpClientMiddlewareCallback',
+    'HttpClientCallback'
 ]
