@@ -4,22 +4,19 @@ from asyncio import AbstractEventLoop
 import urllib.parse
 from ssl import SSLContext
 from typing import (
-    Any,
+    AsyncIterable,
     Iterable,
     List,
-    Mapping,
     Optional,
-    Type,
+    Tuple,
     Union
 )
 
-from bareutils.compression import Decompressor
-from baretypes import Header, Content
-
-from .requester import RequestHandler
-from .acgi import connect
-from .constants import DEFAULT_DECOMPRESSORS, DEFAULT_PROTOCOLS
+from .acgi import connect, RequestHandler
+from .constants import DEFAULT_PROTOCOLS
+from .middleware import HttpClientMiddlewareCallback
 from .ssl_contexts import DEFAULT_CIPHERS, DEFAULT_OPTIONS
+from .response import Response
 
 
 class HttpClient:
@@ -30,19 +27,19 @@ class HttpClient:
             url: str,
             *,
             method: str = 'GET',
-            headers: Optional[List[Header]] = None,
-            content: Optional[Content] = None,
+            headers: Optional[List[Tuple[bytes, bytes]]] = None,
+            body: Optional[AsyncIterable[bytes]] = None,
             loop: Optional[AbstractEventLoop] = None,
             h11_bufsiz: int = 8096,
             cafile: Optional[str] = None,
             capath: Optional[str] = None,
             cadata: Optional[str] = None,
             ssl_context: Optional[SSLContext] = None,
-            decompressors: Optional[Mapping[bytes, Type[Decompressor]]] = None,
             protocols: Iterable[str] = DEFAULT_PROTOCOLS,
             ciphers: Iterable[str] = DEFAULT_CIPHERS,
             options: Iterable[int] = DEFAULT_OPTIONS,
-            connect_timeout: Optional[Union[int, float]] = None
+            connect_timeout: Optional[Union[int, float]] = None,
+            middleware: Optional[List[HttpClientMiddlewareCallback]] = None
     ) -> None:
         """Make an HTTP client.
 
@@ -56,35 +53,19 @@ class HttpClient:
         async def main(url: str) -> None:
             async with HttpClient(url, method='GET') as response:
                 print(response)
-                if response['status_code'] == 200 and response['more_body']:
-                    async for part in response['body']:
+                if response.status == 200 and response.body is not None:
+                    async for part in response.body:
                         print(part)
 
         asyncio.run(main('https://docs.python.org/3/library/cgi.html'))
         ```
 
-        The `response` is a mapping with the following content:
-
-        - **`type`** (_Unicode string_) - Currently the only response is
-        `"http.response"`.
-        - **`acgi["version"]`** (_Unicode string_) - Version of the ACGI spec.
-        - **`http_version`** (_Unicode string_) - One of `"1.0"`, `"1.1"` or `"2"`.
-        - **`stream_id`** (_int_) - The HTTP/2 stream id, otherwise None.
-        - **`status_code`** (_int_) - The HTTP status code.
-        - **`headers`** (_Iterable[[byte string, byte string]]_) - A iterable of [name,
-        value] two-item iterables, where name is the header name, and value is the
-        header value. Order must be preserved in the HTTP response. Header names
-        must be lowercased. Optional; defaults to an empty list. Pseudo headers
-        (present in HTTP/2 and HTTP/3) must not be present.
-        - **`more_body`** (_bool_) - Signifies if the body has more content.
-        - **`body`** (_AsyncIterable[byte string]_) - The body content.
-
         Args:
             url (str): The url
             method (str, optional): The HTTP method. Defaults to 'GET'.
-            headers (Optional[List[Header]], optional): The headers. Defaults to
+            headers (Optional[List[Tuple[bytes, bytes]]], optional): The headers. Defaults to
                 None.
-            content (Optional[Content], optional): The body content. Defaults to
+            body (Optional[AsyncIterable[bytes]], optional): The body content. Defaults to
                 None.
             loop (Optional[AbstractEventLoop], optional): The optional asyncio
                 event loop. Defaults to None.
@@ -99,8 +80,6 @@ class HttpClient:
                 DER-encoded certificates. Defaults to None.
             ssl_context (Optional[SSLContext], optional): An ssl context to be
                 used instead of generating one from the certificates.
-            decompressors (Optional[Mapping[bytes, Type[Decompressor]]], optional):
-                The decompressors. Defaults to None.
             protocols (Iterable[str], optional): The supported protocols. Defaults
                 to DEFAULT_PROTOCOLS.
             ciphers (Iterable[str], optional): The supported ciphers. Defaults
@@ -109,34 +88,49 @@ class HttpClient:
                 to DEFAULT_OPTIONS.
             connect_timeout (Optional[Union[int, float]], optional): The number
                 of seconds to wait for the connection. Defaults to None.
+            middleware (Optional[List[HttpClientMiddlewareCallback]], optional):
+                Optional middleware. Defaults to None.
         """
-        self.url = urllib.parse.urlparse(url)
+        parsed_url = urllib.parse.urlparse(url)
+        if parsed_url.hostname is None:
+            raise ValueError('no hostname in url: ' + url)
+        self.scheme = parsed_url.scheme
+        self.netloc = parsed_url.netloc
+        self.hostname = parsed_url.hostname
+        self.path = parsed_url.path
+        self.port = parsed_url.port
+
         self.method = method
         self.headers = headers
-        self.content = content
+        self.body = body
         self.loop = loop
         self.h11_bufsiz = h11_bufsiz
         self.cafile = cafile
         self.capath = capath
         self.cadata = cadata
         self.ssl_context = ssl_context
-        self.handler: Optional[RequestHandler] = None
-        self.decompressors = decompressors or DEFAULT_DECOMPRESSORS
         self.protocols = protocols
         self.ciphers = ciphers
         self.options = options
         self.connect_timeout = connect_timeout
+        self.middleware = middleware or []
 
-    async def __aenter__(self) -> Mapping[str, Any]:
+        self.handler: Optional[RequestHandler] = None
+
+    async def __aenter__(self) -> Response:
         self.handler = RequestHandler(
-            self.url,
+            self.netloc,
+            self.scheme,
+            self.path,
             self.method,
             self.headers,
-            self.content,
-            self.decompressors
+            self.body,
+            self.middleware
         )
         response = await connect(
-            self.url,
+            self.scheme,
+            self.hostname,
+            self.port,
             self.handler,
             cafile=self.cafile,
             capath=self.capath,
